
#ifndef ___rss_marshal_MARSHAL_H__
#define ___rss_marshal_MARSHAL_H__

#include	<glib-object.h>

G_BEGIN_DECLS

/* VOID:VOID (./rss-marshal.list:1) */
#define _rss_marshal_VOID__VOID	g_cclosure_marshal_VOID__VOID

G_END_DECLS

#endif /* ___rss_marshal_MARSHAL_H__ */

