//	This file is part of FeedReader.
//
//	FeedReader is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	FeedReader is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with FeedReader.  If not, see <http://www.gnu.org/licenses/>.

public class FeedReader.Enclosure : GLib.Object {

	private string m_article_id;
	private string m_url;
	private EnclosureType m_type;

	public Enclosure(string article_id, string url, EnclosureType type)
	{
		m_article_id = article_id;
		m_url = url;
		m_type = type;
	}

	public string get_url()
	{
		return m_url;
	}

	public EnclosureType get_enclosure_type()
	{
		return m_type;
	}
}
