.fr-sidebar-separator {
	color: alpha(#333, 0.2);
}
.fr-sidebar {
	background-color: shade (#DEDEDE, 0.92);
	color: mix (#DEDEDE, #333, 0.60);
}
.fr-sidebar-row,
.fr-sidebar-feed {
	padding: 0 6px;
}
.fr-sidebar-row label,
.fr-sidebar-feed label {
	padding: 4px;
}
.fr-sidebar-row,
.fr-sidebar-feed {
	border-color: transparent;
	border-style: solid;
	border-width: 1px 0 1px 0;
	text-shadow: 0 1px alpha(#fff, 0.4);
}
.fr-sidebar-row:hover,
.fr-sidebar-feed:hover {
	background-color: shade (#DEDEDE, 1.00);
}
.fr-sidebar-row:selected,
.fr-sidebar-feed:selected {
	color: mix (#DEDEDE, #333, 0.60);
	background-color: transparent;
	background-image: linear-gradient(to bottom,
					alpha (#000, 0.11),
					alpha (#000, 0.07)
				);
	border-image: linear-gradient(to bottom,
					alpha (#000, 0.25),
					alpha (#fff, 0.30)
				) 1 1 1 1;
	box-shadow: inset 0 1px 0 0 alpha (#000, 0.03),
			inset 0 -1px 0 0 alpha (#000, 0.10);
}
.fr-sidebar-row:backdrop,
.fr-sidebar-feed:backdrop {
	color: mix (#DEDEDE, #333, 0.40);
	-gtk-icon-effect: dim;
}
.fr-sidebar-row:selected:backdrop,
.fr-sidebar-feed:selected:backdrop {
	color: mix (#DEDEDE, #333, 0.40);
	background-image: linear-gradient(to bottom,
					alpha (#000, 0.04),
					alpha (#000, 0.03)
				);
	border-image: linear-gradient(to bottom,
					alpha (#000, 0.15),
					alpha (#fff, 0.25)
				) 1 1 1 1;
}
.fr-sidebar.frame {
	border-width: 1px 0 1px 0;
}
.fr-sidebar .frame {
	border: none;
}
.fr-sidebar-symbolic {
	color: alpha(#333, 0.8);
}
/******************/
/* sidebar footer */
/******************/

.footer {
	background: shade (#DEDEDE, 0.85);
}
.footer button:hover,
.footer-popover {
	background: shade (#DEDEDE, 0.96);
}
.footer button {
	box-shadow: none;
}
